Package: stylo
Type: Package
Title: Stylometric Multivariate Analyses
Version: 0.7.71
Date: 2026-06-16
Authors@R: c(person("Maciej", "Eder", role = c("aut", "cre"), 
           email = "maciejeder@gmail.com"),
             person("Jan", "Rybicki", role = "aut"),
             person("Mike", "Kestemont", role = "aut"),
             person("Steffen", "Pielstroem", role = "aut"))
Author: Maciej Eder [aut, cre],
  Jan Rybicki [aut],
  Mike Kestemont [aut],
  Steffen Pielstroem [aut]
Maintainer: Maciej Eder <maciejeder@gmail.com>
URL: https://github.com/computationalstylistics/stylo
Depends: R (>= 3.0)
Imports: ape, pamr, e1071, class, lattice, tsne, tcltk, tcltk2
Suggests: stringi, networkD3, readr, testthat
Config/testthat/edition: 3
Description: Supervised and unsupervised multivariate methods, supplemented by GUI and some visualizations, to perform various analyses in the field of computational stylistics, authorship attribution, etc. For further reference, see Eder et al. (2016), <https://journal.r-project.org/articles/RJ-2016-007/index.html>. You are also encouraged to visit the Computational Stylistics Group's website <https://computationalstylistics.github.io/>, where a reasonable amount of information about the package and related projects are provided.
License: GPL (>= 3)
NeedsCompilation: no
Packaged: 2026-06-16 10:48:53 UTC; m
Repository: CRAN
Date/Publication: 2026-06-16 12:00:02 UTC
Built: R 4.5.2; ; 2026-06-16 14:02:04 UTC; unix
